unit Main;

interface

uses
  Windows, OpenGL12, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TMainForm = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    FRC: HGLRC;
    FAngle: Integer;
    FMainList: Cardinal;
    FNeedSetup: Boolean;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure Setup;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

function CheckRes: Boolean;

var
  Error: TGLEnum;
  
begin
  Error := glGetError;
  Result := Error = GL_NO_ERROR;
  if not Result then
    ShowMessage('Error: ' + gluErrorString(Error));
end;


procedure TMainForm.WMEraseBkgnd(var Message: TWMEraseBkgnd);

begin
  Message.Result := 1;
end;

procedure TMainForm.FormCreate(Sender: TObject);

begin
  FNeedSetup := True;
end;

procedure TMainForm.FormDestroy(Sender: TObject);

begin
  wglMakeCurrent(Canvas.Handle, FRC);
  glDeleteLists(FMainList, 1);
  wglMakeCurrent(0, 0);
  wglDeleteContext(FRC);
end;

procedure TMainForm.FormPaint(Sender: TObject);

begin
  if FNeedSetup then Setup;
  wglMakeCurrent(Canvas.Handle, FRC);
  // clear "background" (color and depth buffer)
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  // viewer position and direction
  gluLookAt(0, 0, 6, 0, 0, -10, 0, 1, 0);

  // a bit animation
  glRotatef(4 * FAngle, 0, 1, 0);
  glRotatef(120, 1, 0, 0);
  glCallList(FMainList);
  SwapBuffers(Canvas.Handle);
  wglMakeCurrent(0, 0);;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);

begin
  Inc(FAngle, 1);
  if FAngle >= 360 then FAngle := 0;
  Repaint;
end;

function BuildObject: Cardinal;

// creates the main (and only) display list

var
  Quadric: PGLUquadricObj;

begin
  Result := glGenLists(1);
  glNewList(Result, GL_COMPILE);
    Quadric := gluNewQuadric;
    gluQuadricDrawStyle(Quadric, GLU_FILL);
    gluQuadricNormals(Quadric, GLU_SMOOTH);
    gluQuadricOrientation(Quadric, GLU_OUTSIDE);
    glRotatef(-90, 1, 0, 0);
    glTranslated(0, 0, -0.5);
    gluCylinder(Quadric, 1, 0.5, 1, 20, 20);
    glPushMatrix;
    glTranslatef(0, 0, 1);
    gluDisk(Quadric, 0, 0.5, 20, 20);
    glPopMatrix;
    gluQuadricOrientation(Quadric, GLU_INSIDE);
    gluDisk(Quadric, 0, 1, 20, 20);
    gluDeleteQuadric(Quadric);
  glEndList;
  if not CheckRes then Halt;
end;

procedure TMainForm.Setup;

const
  Light1: array[0..3] of Single = (-30, 30, 30, 1);
  Light2: array[0..3] of Single = (50, 100, 50, 1);

  Ambient: array[0..3] of Single = (0.2, 0.2, 0.2, 1);
  Diffuse1: array[0..3] of Single = (1, 1, 1, 1);
  Diffuse2: array[0..3] of Single = (0, 0.7, 0.37, 1);
  Diffuse3: array[0..3] of Single = (0.5, 0.5, 0.5, 1);
  Specular: array[0..3] of Single = (1, 1, 1, 1);

begin
  FNeedSetup := False;
  FRC := CreateRenderingContext(Canvas.Handle, [opDoubleBuffered], 32, 0, 0, 0, 0);
  wglMakeCurrent(Canvas.Handle, FRC);

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);

  // Lichtquellen einrichten
  glEnable(GL_LIGHTING);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, Ord(GL_FALSE));

  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, @Light1);
  glLightfv(GL_LIGHT0, GL_AMBIENT, @Ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @Diffuse1);
  glLightfv(GL_LIGHT0, GL_SPECULAR, @Specular);

  glEnable(GL_LIGHT1);
  glLightfv(GL_LIGHT1, GL_POSITION, @Light2);
  glLightfv(GL_LIGHT1, GL_AMBIENT, @Ambient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @Diffuse2);
  glLightfv(GL_LIGHT1, GL_SPECULAR, @Specular);

  // Display Liste fr die Szene erstellen (hier ein Zylinder)
  FMainList := BuildObject;

  glMateriali(GL_FRONT, GL_SHININESS, 128);
  glMaterialfv(GL_FRONT, GL_AMBIENT, @Ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, @Diffuse3);
  glMaterialfv(GL_FRONT, GL_SPECULAR, @Specular);


  glViewport(0, 0, ClientWidth, ClientHeight);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  if ClientHeight = 0 then ClientHeight := 1;
  gluPerspective(35, ClientWidth / ClientHeight, 1, 100);

  if not CheckRes then Halt;
  // background color
  glClearColor(0.5, 0.5, 0.5, 1);
end;

procedure TMainForm.FormResize(Sender: TObject);
// sets the viewport of the rendering context
begin
  if FRC <> 0 then
  begin
    ActivateRenderingContext(Canvas.Handle, FRC);
    glViewport(0, 0, ClientWidth, ClientHeight);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity;
    if ClientHeight = 0 then ClientHeight := 1;
    gluPerspective(35, ClientWidth / ClientHeight, 1, 100);
    DeactivateRenderingContext;
  end;
end;

end.
